﻿//-----------------------------------------------------------------------------
// Tower.cs
// This is the tower class.  Each different type of tower inherits this type.
// Contains variables and functions that define basic tower properties, such
// as cost, level, the current target etc.
//
// Authors: John Karczewski, Kelsey Schlarman
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using SpriteSheetRuntime;
using ZuneDefense;

namespace ZuneDefense
{
    public abstract class Tower : DrawableZuneDefenseComponent
    {
        #region Fields
        protected static readonly int width = 2 * GameBoard.TileWidthInPixels;
        protected static readonly int height = 2 * GameBoard.TileHeightInPixels;

        protected Enemy closestEnemy;
        protected double timeUntilNextShot;
        protected BulletCollection bulletCol;
        protected String bulletType;

        public bool isSplash;
        public bool isSlowing;
        protected bool shootsAir;
        protected bool shootsGround;

        public float[] fireRates;
        public int[] costs;
        public int[] ranges;
        public int[] damages;
        public int[] splashDistance;
        public float[] slow;
        public float[] splash;
        

        public int level;
        public int maxLevel;

        protected String name;
        protected String description;

        #endregion

        #region Properties

        public int TextureIndex
        {
            get { return textureIndex; }
        }


        public String Name
        {
            get { return name; }
        }


        public String Description
        {
            get { return description; }
        }

        public int Damage
        {
            get { return damages[level]; }
        }

        public int SplashDistance
        {
            get { return splashDistance[level]; }
        }

        public float Range
        {
            get { return ranges[level]; }
        }

        public float FireRate
        {
            get { return fireRates[level]; }
        }

        public float Splash
        {
            get { return splash[level]; }
        }

        public int Cost
        {
            get { return costs[level]; }
        }

        public String BulletType
        {
            get { return bulletType; }
        }

        public bool ShootsAir
        {
            get { return shootsAir; }
        }

        public bool ShootsGround
        {
            get { return shootsGround; }
        }

        #endregion

        #region Initialization


        /// <summary>
        /// Constructs a new tower
        /// </summary>
        protected Tower(SpriteSheet spriteSheet, SpriteBatch spriteBatch, Vector2 position, string NAME, int[] DAMAGES, int[] RANGES, float[] FIRERATES, float[] SPLASH, int[] SPLASHDISTANCE, int[] COSTS, int MAXLEVEL, String DESCRIPTION, string BULLETTYPE)
            : base(spriteSheet, spriteBatch, spriteSheet.GetIndex("ExampleTower"), position, new Rectangle(0, 0, width, height))
        {
            this.bulletType = BULLETTYPE;
            isSplash = false;
            isSlowing = false;

            name = NAME;
            description = DESCRIPTION;
            maxLevel = MAXLEVEL;

            fireRates = FIRERATES;
            ranges = RANGES;
            damages = DAMAGES;
            costs = COSTS;
            splash = SPLASH;
            splashDistance = SPLASHDISTANCE;

            level = 0;

            this.bulletCol = new BulletCollection();
            closestEnemy = null;

            slow = new float[maxLevel];
            shootsAir = true;
            shootsGround = true;
        }

        #endregion

        #region Draw and Update

        /// <summary>
        /// Determines if the tower has a valid enemey targeted.
        /// </summary>

        public bool hasEnemy()
        {
            if (closestEnemy != null && closestEnemy.Life > 0)
            {
                if (Vector2.Distance(closestEnemy.CenterPixelPosition, CenterPixelPosition) < Range)
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Upgrades the tower's level
        /// </summary>
        public void upgrade()
        {
            if (this.canUpgrade())
            {
                level++;
            }
        }

        /// <summary>
        /// Determines if a tower is at the max level.
        /// </summary>

        public bool canUpgrade()
        {
            return (level < maxLevel);
        }

        /// <summary>
        /// Called when the tower needs to be drawn.
        /// </summary>
        /// <param name="gameTime"></param>
        /// <param name="camera"></param>
        public override void Draw(GameTime gameTime, Camera camera)
        {
            bounds.Location = spriteSheet.SourceRectangle(textureIndex).Location;
            bounds.Offset(CurrentSpriteDirection() * width, 0);
            spriteBatch.Draw(spriteSheet.Texture, new Vector2(pixelPosition.X, pixelPosition.Y + camera.Position.Y), bounds, Color.White);
            bulletCol.Draw(gameTime, camera);
        }

        /// <summary>
        /// Called when tower needs to be updated, sets enemy to closest Enemey
        /// </summary>
        /// <param name="enemy"></param>
        public void setEnemy(Enemy enemy)
        {
            closestEnemy = enemy;
        }

        /// <summary>
        /// Handles tower targeting, and bullet updating
        /// </summary>
        /// <param name="gameTime"></param>

        public override void Update(GameTime gameTime)
        {
            if (closestEnemy != null && closestEnemy.Life > 0)
            {
                direction = Vector2.Normalize(Vector2.Subtract(closestEnemy.CenterPixelPosition, CenterPixelPosition));
                if (Vector2.Distance(closestEnemy.CenterPixelPosition, CenterPixelPosition) < Range)
                {
                    if (timeUntilNextShot <= 0)
                    {
                        timeUntilNextShot = FireRate;
                        bulletCol.Add(new Bullet(spriteSheet, spriteBatch, this, closestEnemy));
                    }
                    else
                        timeUntilNextShot -= gameTime.ElapsedGameTime.TotalSeconds;
                }
                else
                {
                    closestEnemy = null;
                }
            }
            
            bulletCol.Update(gameTime);
        }
        #endregion
    }
}
